package com.elsevier.example.music.quarkus;

import com.amazonaws.services.lambda.runtime.Context;
import com.amazonaws.services.lambda.runtime.RequestHandler;
import com.elsevier.example.music.quarkus.codec.ChartEntryDecoder;
import com.elsevier.example.music.quarkus.codec.CsvChartEntryDecoder;
import com.elsevier.example.music.quarkus.reader.ChartEntryReaderFactory;
import com.elsevier.example.music.quarkus.reader.S3ChartEntryReaderFactory;
import com.elsevier.example.music.quarkus.writer.ChartEntryWriterFactory;
import com.elsevier.example.music.quarkus.writer.DynamoChartEntryWriterFactory;


public class Handler implements RequestHandler<Void, Void> {
    private static final String INPUT_S3_BUCKET = "dev-day-temp";
    private static final String INPUT_S3_KEY = "music.csv";
    private static final String OUTPUT_DYNAMO_TABLE = "dev-day-experiment";
    private static final int OUTPUT_BATCH_SIZE = 25;

    private final ChartEntryCopier chartEntryCopier;

    public Handler() {
        ChartEntryDecoder chartEntryDecoder = new CsvChartEntryDecoder();
        ChartEntryReaderFactory chartEntryReaderFactory = new
                S3ChartEntryReaderFactory(INPUT_S3_BUCKET, INPUT_S3_KEY, chartEntryDecoder);
        ChartEntryWriterFactory chartEntryWriterFactory = new DynamoChartEntryWriterFactory(OUTPUT_DYNAMO_TABLE, OUTPUT_BATCH_SIZE);
        chartEntryCopier = new ChartEntryCopier(chartEntryReaderFactory, chartEntryWriterFactory);
    }

    @Override
    public Void handleRequest(Void input, Context context) {
        chartEntryCopier.copyEntries();
        return null;
    }
}
