package com.elsevier.example.music.quarkus.codec;

import com.elsevier.example.music.quarkus.ChartEntry;
import com.opencsv.bean.CsvBindByName;
import com.opencsv.bean.CsvToBean;
import com.opencsv.bean.CsvToBeanBuilder;
import lombok.Data;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.stream.Stream;

public class CsvChartEntryDecoder implements ChartEntryDecoder {
    private static final DateTimeFormatter CSV_DATE_FORMATTER = DateTimeFormatter.ofPattern("yyyyMMdd");

    @Data
    public static class CsvChartEntry {
        @CsvBindByName(column = "Position")
        int position;
        @CsvBindByName(column = "Artist")
        String artist;
        @CsvBindByName(column = "Song")
        String song;
        @CsvBindByName(column = "ChartDate")
        String date;
    }

    public Stream<ChartEntry> decode(InputStream inputStream) {
        CsvToBean<CsvChartEntry> csvToBean = new CsvToBeanBuilder(new InputStreamReader(inputStream))
            .withType(CsvChartEntry.class)
            .build();
        return csvToBean.stream()
             .map(CsvChartEntryDecoder::toChartEntry);
    }

    private static ChartEntry toChartEntry(CsvChartEntry csvChartEntry) {
        LocalDate date = LocalDate.parse(csvChartEntry.getDate(), CSV_DATE_FORMATTER);
        return new ChartEntry(csvChartEntry.getArtist(), csvChartEntry.getSong(), date, csvChartEntry.getPosition());
    }
}
